﻿using System;
using System.Data;
using Wisej.Web;

namespace Wisej_DGVVirtualMode
{
	public partial class Page1 : Page
	{
		private const int PAGE_SIZE = 25;

		public Page1()
		{
			InitializeComponent();
		}

		private void Page1_Load(object sender, System.EventArgs e)
		{
			// number of rows per block (better not to change the default).
			this.dataGridView1.BlockSize = 50;

			var rowCount = DB.GetEmployeeRowCount();
			var pageCount = rowCount / PAGE_SIZE;

			// virtual mode grid.
			this.dataGridView1.RowCount = rowCount;

			// paged grid.
			this.trackBar1.Maximum = pageCount - 1;
			LoadPage(0);
		}

		#region Legacy Paged DataGridView

		private void trackBar1_ValueChanged(object sender, EventArgs e)
		{
			LoadPage(this.trackBar1.Value);
		}

		private void LoadPage(int pageIndex)
		{
			this.dataGridView2.DataSource = DB.ReadEmployeeRows(pageIndex * PAGE_SIZE, PAGE_SIZE);

			Log($"Loaded page {pageIndex} from the database", this.listBox2);
		}

		#endregion

		#region Virtual Mode DataGridView

		private ArtistCache _cache = new ArtistCache(50, 3);

		private void dataGridView1_DataRead(object sender, DataGridViewDataReadEventArgs e)
		{
			this._cache.LoadCache(e.FirstIndex, e.LastIndex - e.FirstIndex + 1);
		}

		private void dataGridView1_CellValueNeeded(object sender, DataGridViewCellValueEventArgs e)
		{
			e.Value = this._cache.GetValue(e.RowIndex, e.ColumnIndex);

			Log($"Pulled cell {e.RowIndex}, {e.ColumnIndex} from the cache", this.listBox1);
		}

		#endregion

		#region Logging

		private void Log(string text, ListBox listBox)
		{
			listBox.SelectedIndex = listBox.Items.Add(text);
		}

		private void listBox2_ToolClick(object sender, ToolClickEventArgs e)
		{
			this.listBox2.Items.Clear();
		}

		private void listBox1_ToolClick(object sender, ToolClickEventArgs e)
		{
			this.listBox1.Items.Clear();
		}

		#endregion
	}
}
